import ConfigParser

CONFIG_PATH = '/etc/sensors/config'

def load_config(path=CONFIG_PATH):
    cfg = ConfigParser.RawConfigParser()
    with open(path, 'r') as f:
        cfg.readfp(f)
    return cfg



import logging

logging.basicConfig()
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

class PID(object):
    def __init__(self, setpoint, Kp, Ki, Kd):
        self.Kp = Kp
        self.Ki = Ki
        self.Kd = Kd
        self.last_prop = 0
        self.last_pv = None
        self.min_output = 0.15
        self.reset(setpoint)

    def setLastValue(self, val):
        self.last_pv = val

    def reset(self, setpoint):
        self.setpoint = setpoint
        self.last_diff = 0
        self.error_integral = 0

    def update(self, pv):
        if self.setpoint is None:
            return 0

        error = self.setpoint - pv

        # Constrain total error to avoid windup problems
        self.error_integral += self.Ki * error
        self.error_integral = max(min(self.error_integral, 1), -1)
        diff = pv - self.last_pv

        self.last_diff = self.Kd * diff
        self.last_prop = self.Kp * error
        self.last_pv = pv

        out = self.Kp * error + self.error_integral - self.Kd * diff
        if out < self.min_output:
            rv = 0
        else:
            rv = max(min(out, 1), self.min_output)
        logger.debug("Unconstrained pid return value %f -> %f", out, rv)

        return rv



#!/usr/bin/env python

import datetime
from flask import Flask, jsonify, request, g

import model
from scheduler import SchedulerTemperaturePolicy
from config import load_config

app = Flask(__name__)

def get_conf():
    if not hasattr(g, 'conf'):
        g.conf = load_config()
    return g.conf

def get_db():
    conf = get_conf()
    if not hasattr(g, 'db'):
        g.db = model.db_connect(
            conf.get('heating', 'scheduler_db_host'),
            conf.get('heating', 'scheduler_db_name'),
            conf.get('heating', 'scheduler_db_user'),
            conf.get('heating', 'scheduler_db_password'))
    return g.db

@app.teardown_appcontext
def close_db(error):
    if hasattr(g, 'db'):
        g.db.close()

def get_tgt_override_json(db):
    target_override_obj = model.TargetOverride.from_db(db)
    if target_override_obj:
        return {
            'until': target_override_obj.end.strftime("%Y-%m-%dT%H:%M"),
            'temp': target_override_obj.temp
            }
    else:
        return {}

@app.route("/summary")
def get_summary():
    now = datetime.datetime.now()
    db = get_db()

    schedule = model.FullSchedule.from_db(db)
    tgt_override = get_tgt_override_json(db)
    tgt_override_obj = model.TargetOverride.from_db(db)
    scheduler = SchedulerTemperaturePolicy(
        schedule, tgt_override_obj)
    target = scheduler.target(now)

    today = scheduler.get_day(now.weekday())
    today = [{'time': starttime.strftime("%H:%M"), 'temp': temp}
             for (starttime, temp) in today]
    temp = model.get_last_temperature(db)
    cached_temp = temp.temp if temp else None

    db.commit()
    result = {
        'target': target[0],
        'target_entry': target[1],
        'target_overridden': target[1] == -2,
        'current': cached_temp,
        'server_day_of_week': now.weekday(),
        'today': today,
        'target_override': tgt_override,
        }
    return jsonify(result)

@app.route("/target_override", methods=["DELETE"])
def remove_target_override():
    """Removes the target override if in place."""
    db = get_db()
    model.TargetOverride.clear_from_db(db)
    db.commit()
    return ('', 204)

@app.route("/temperature", methods=["POST"])
def update_cached_temperature():
    """Updates the currently-cached temperature value.

    Expects request values 'when' (datetime) and 'temp'"""
    try:
        temp = float(request.values['temp'])
        when = datetime.datetime.strptime(
            request.values['when'], "%Y-%m-%dT%H:%M:%S")
    except ValueError:
        return ('', 400)

    db = get_db()
    model.update_last_temperature(db, when, temp)
    db.commit()
    return ('', 200)

@app.route("/schedule")
def get_schedule():
    db = get_db()
    full_schedule = model.FullSchedule.from_db(db)
    json_schedule = {}
    for dow in range(7):
        json_schedule[dow] = []
    for entry in full_schedule:
        entry_dow, entry_start, entry_temp = entry
        dow = json_schedule[entry_dow]
        dow.append({
            'time': entry_start.strftime('%H:%M'),
            'temp': entry_temp
            })
    tgt_override = get_tgt_override_json(db)
    db.commit()
    return jsonify({
        'schedule': json_schedule,
        'target_override': tgt_override
        })

@app.route("/schedule/new_entry", methods=["POST"])
def add_schedule_entry():
    db = get_db()
    try:
        print request.values
        time = datetime.datetime.strptime(request.values['time'], "%H:%M")
        time = time.time()
        day = int(request.values['day'])
        if not (day >= 0 and day < 7):
            raise ValueError("Day of week must be in range 0 to 7")
        temp = float(request.values['temp'])
        if not (temp >= 0 and temp < 35):
            raise ValueError("Target tempt must be in range 0 to 35")
    except ValueError:
        return ('', 400)
    model.FullSchedule.create_entry(db, day, time, temp)
    db.commit()
    return ''

@app.route("/schedule/delete_entry", methods=["POST"])
def remove_schedule_entry():
    db = get_db()
    try:
        time = datetime.datetime.strptime(request.values['time'], "%H:%M")
        time = time.time()
        day = int(request.values['day'])
        if not (day >= 0 and day < 7):
            raise ValueError("Day of week must be in range 0 to 7")
    except ValueError:
        return ('', 400)
    model.FullSchedule.delete_entry(db, day, time)
    db.commit()
    return ''

@app.route("/target_override", methods=["POST"])
def set_target_override():
    """Set a target override for a duration specified.

    Specify 'temp' and at least one of 'days', 'hours', or 'mins'.
    Returns a JSON object containg 'temp' (the newly-accepted target)
    and 'until' which is a %H:%M-formatted time at which the override
    ends (so the client doesn't have to do a time calculation for this).
    """
    try:
        secs = 0
        if 'days' in request.form:
            secs += int(request.form['days']) * 60 * 60 * 24
        if 'hours' in request.form:
            secs += int(request.form['hours']) * 60 * 60
        if 'mins' in request.form:
            secs += int(request.form['mins']) * 60
        if not secs:
            return # XXX exception

        duration = datetime.timedelta(0, secs)
        temp = float(request.form['temp'])
    except ValueError:
        return '', 400
    now = datetime.datetime.now()
    end = now + duration

    db = get_db()
    override = model.TargetOverride(end, temp)
    override.save(db)
    db.commit()

    return ('', 200)

if __name__ == "__main__":
    app.run()



import datetime
import logging

logging.basicConfig()
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

class PWM(object):
    def __init__(self, dutycycle, period):
        self.dutycycle = dutycycle
        self.period = period
        self.active = False
        self.periodBegin = None

    def setDutyCycle(self, dutycycle):
        self.dutycycle = dutycycle

    def update(self, now):
        # Begin new cycle?
        if self.periodBegin is None or \
           self.periodBegin + self.period <= now:
            logger.debug("Beginning PWM new cycle @ %s", str(now))
            self.periodBegin = now
            if self.dutycycle > datetime.timedelta(0, 0):
                self.on()
                self.active = True
            return

        # End of 'on' cycle?
        if (self.periodBegin + self.dutycycle) <= now:
            if self.dutycycle <= self.period and self.active:
                logger.debug("End of PWM duty cycle @ %s", str(now))
                self.off()
                self.active = False
            return

    def on(self):
        raise NotImplementedError
    def off(self):
        raise NotImplementedError





#!/usr/bin/env python

import datetime
import logging
import argparse
import json
import time

import paho.mqtt.client as mqtt

import config
import pwm
import pid

logging.basicConfig()
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

# Settings:

# The period of one on-off cycle when maintaining/monitoring the average
# temperature.
PWM_PERIOD = datetime.timedelta(0, 600)
PWM_MEASUREMENT_PERIOD = PWM_PERIOD

PID_KP = 2.8
PID_KI = 0.3
PID_KD = 1.8

# The width of the 'target zone'; we'll try to adjust within this and
# learn how much heating is required, but outside that we'll either just
# enable or disable heating.
TARGET_ZONE_WIDTH = 0.6

# Time after which a temperature reading is considered too old to use
STALE_THRESHOLD = datetime.timedelta(0, 600)

# Time after which we might re-issue the same command to the boiler
REISSUE_TIMEOUT = datetime.timedelta(0, 120)

BOILER_COMMAND_OFF = 'X'
BOILER_COMMAND_ON = 'O'

class BoilerControl(object):
    def __init__(self, thermostat_id):
        self.thermostat_id = thermostat_id

    def command(self, cmd):
        raise NotImplementedError

class MqttBoilerControl(BoilerControl):
    """Control boiler using MQTT commands."""
    def __init__(self, thermostat_id, mqttc, zone_demand_topic):
        self.mqttc = mqttc
        self.zone_demand_topic = zone_demand_topic
        super(MqttBoilerControl, self).__init__(thermostat_id)

    def command(self, cmd):
        self.mqttc.publish(self.zone_demand_topic, json.dumps({
            'thermostat': self.thermostat_id,
            'command': cmd}))

class BoilerPWM(pwm.PWM):
    def __init__(self, state, dutycycle, pwmperiod):
        self.state = state
        super(BoilerPWM, self).__init__(dutycycle, pwmperiod)

    def on(self):
        self.state.boiler_command(BOILER_COMMAND_ON)

    def off(self):
        self.state.boiler_command(BOILER_COMMAND_OFF)

# Global state:

class State(object):
    def __init__(self, boiler_control):
        self.now = None
        self.lastCommand = None
        self.lastReading = None
        self.state = None

        self.pwmDutyCycle = None
        self.pwmBoilerCycle = None
        self.pwmMeasurementBegin = None

        self.pid = pid.PID(None, PID_KP, PID_KI, PID_KD)

        self.targetTemp = None
        self.boilerControl = boiler_control

    def update_target_temperature(self, target):
        if target != self.targetTemp:
            logger.info("Target temperature changed from %s to %s",
                        str(self.targetTemp), target)
            self.targetTemp = target
            self.pid.reset(target)

    def update_state(self, new_state):
        if self.state != new_state:
            oldname, _, _ = ("None", None, None) if not self.state \
                            else state_fns[self.state]
            name, _, enter = state_fns[new_state]
            logger.info("Transitioning from state %s to %s", oldname, name)
            self.state = new_state

            # If it has an entry function, run it:
            if enter:
                enter(self)

    def boiler_command(self, cmd):
        if self.lastCommand is None or \
           self.now - self.lastCommand.when > REISSUE_TIMEOUT or \
           self.lastCommand.cmd != cmd:
            logger.info("Issuing boiler command %s to %s",
                        cmd, str(self.boilerControl))
            if self.boilerControl:
                self.boilerControl.command(cmd)
            self.lastCommand = TempCommand(self.now, cmd)

    def last_temperature(self):
        return None if self.lastReading is None \
               else self.lastReading.temp

    def target_zone_max(self):
        return self.targetTemp + TARGET_ZONE_WIDTH / 2
    def target_zone_min(self):
        return self.targetTemp - TARGET_ZONE_WIDTH / 2

    def temperature_reading_stale(self):
        return self.targetTemp is None or \
               self.lastReading is None or \
               self.lastReading.when < (self.now - STALE_THRESHOLD)

    def update_duty_cycle(self, dc):
        self.pwmDutyCycle = datetime.timedelta(0, PWM_PERIOD.total_seconds() * dc)
        if not self.pwmBoilerCycle:
            self.pwmBoilerCycle = BoilerPWM(self, self.pwmDutyCycle, PWM_PERIOD)
        else:
            self.pwmBoilerCycle.setDutyCycle(self.pwmDutyCycle)

    def update_temperature(self, temp):
        self.lastReading = TempReading(self.now, temp)
        logger.debug("Temperature update: %s", str(temp))

class TempReading(object):
    def __init__(self, when, temp):
        self.when = when
        self.temp = temp

class TempCommand(object):
    def __init__(self, when, cmd):
        self.when = when
        self.cmd = cmd

# State implementations:

(MODE_OFF, MODE_ON, MODE_PWM, MODE_STALE) = range(4)

def mode_off(state):
    # If we moved below the top of the target zone, set the state
    # accordingly
    if state.last_temperature() < state.target_zone_min():
        state.update_state(MODE_ON)
    elif state.last_temperature() < state.targetTemp:
        state.update_state(MODE_PWM)
    else:
        state.boiler_command(BOILER_COMMAND_OFF)

def mode_on(state):
    # If for some reason (unexpectedly) we ended up above the target zone,
    # switch heating off:
    if state.last_temperature() > state.target_zone_max():
        logger.warn("Temperature got above target zone from MODE_ON")
        state.boiler_command(BOILER_COMMAND_OFF)
        state.update_state(MODE_OFF)
        return

    # If we are within target zone, switch to the PID controller:
    if state.last_temperature() >= (state.targetTemp - TARGET_ZONE_WIDTH / 2):
        logger.info("Temperature in target zone")
        state.boiler_command(BOILER_COMMAND_OFF)
        state.update_state(MODE_PWM)
        return

    # Otherwise, just keep the boiler going:
    state.boiler_command(BOILER_COMMAND_ON)

def mode_pwm_enter(state):
    # Record the temperature target when we enter, so that we know whether
    # to keep state on exit (i.e. if we messed up and temperature left the
    # target zone even though the target didn't change).
    state.pwmBoilerCycle = BoilerPWM(state, datetime.timedelta(0), PWM_PERIOD)
    state.pid.setLastValue(state.last_temperature())

def mode_pwm(state):
    # If we go outside the target zone, switch modes:
    if state.last_temperature() > state.target_zone_max():
        logger.error("Exceeded target zone in PWM mode")
        state.update_state(MODE_OFF)
        return
    if state.last_temperature() < state.target_zone_min():
        logger.error("Dropped below target zone in PWM mode")
        state.update_state(MODE_ON)
        return

    # New measurement cycle?
    if state.pwmMeasurementBegin is None or \
       state.pwmMeasurementBegin + PWM_MEASUREMENT_PERIOD < state.now:
        state.pwmMeasurementBegin = state.now
        # Adjust duty cycle:
        pid_output = state.pid.update(state.last_temperature())
        state.update_duty_cycle(pid_output)

        logger.debug("PID ouutput: %f", pid_output)
        logger.debug("PID internals: prop %f, int %f, diff %f",
                     state.pid.last_prop, state.pid.error_integral,
                     state.pid.last_diff)
        logger.debug("New measurement cycle started")

    state.pwmBoilerCycle.update(state.now)

def mode_stale(state):
    # Ideally we'd cycle the zone periodically to avoid freezing, given we
    # don't have information to do anything better.  For now, just turn the
    # zone off:
    state.boiler_command(BOILER_COMMAND_OFF)

# Mode -> (Mode fn, enter fn)
state_fns = {
    MODE_OFF: ("Off", mode_off, None),
    MODE_ON: ("On", mode_on, None),
    MODE_PWM: ("PWM", mode_pwm, mode_pwm_enter),
    MODE_STALE: ("Stale", mode_stale, None),
    }

def period(state):
    if state.temperature_reading_stale():
        # Don't use stale temperature readings to make decisions:
        state.update_state(MODE_STALE)
    elif state.state is None or state.state == MODE_STALE:
        # If we just a reading after having had stale or no data, pick
        # a starting state from scratch:
        if state.last_temperature() < (state.targetTemp - TARGET_ZONE_WIDTH / 2):
            state.update_state(MODE_ON)
        elif state.last_temperature() > (state.targetTemp + TARGET_ZONE_WIDTH / 2):
            state.update_state(MODE_OFF)
        else:
            state.update_state(MODE_PWM)

    # Now do whatever we're supposed to in this state:
    _, state_fn, _ = state_fns[state.state]
    state_fn(state)

# Temperature reading update:
def on_connect(client, userdata, flags, rc):
    if rc:
        logger.error("Error connecting, rc %d", rc)
        return
    client.subscribe(userdata['tempsensor'])
    client.subscribe(userdata['target_temp'])
    client.publish(userdata['thermostat_status_topic'], json.dumps({
        'thermostat_id': userdata['thermostat_id'],
        'status': 'online'}))

def on_message(client, userdata, msg):
    state = userdata['state']

    def temperature_update():
        data = json.loads(msg.payload)
        if 'temperature' not in data:
            return

        try:
            temp = float(data['temperature'])
        except ValueError:
            return

        now = datetime.datetime.now()
        state.now = now
        state.update_temperature(temp)

        # Print information for debugging/graphing:
        duty_cycle = state.pwmDutyCycle.total_seconds() \
                     if state.pwmDutyCycle else 0
        print now, state.targetTemp, state.lastCommand.cmd, duty_cycle, \
              state.last_temperature(), state.pid.last_prop, \
              state.pid.error_integral, state.pid.last_diff

    def target_update():
        # Payload should be dictionary with key 'target':
        try:
            target = float(json.loads(msg.payload)['target'])
            userdata['state'].update_target_temperature(target)
        except (KeyError, ValueError):
            pass

    if msg.topic == userdata['tempsensor']:
        temperature_update()
    elif msg.topic == userdata['target_temp']:
        target_update()

def maintain_temp(sensor_topic, thermostat_id, dry_run):
    conf = config.load_config()
    mqttc = mqtt.Client()
    if not dry_run:
        boiler_control = MqttBoilerControl(
            thermostat_id, mqttc, conf.get('heating', 'demand_request_topic'))
    else:
        boiler_control = None
    state = State(boiler_control)

    mqttc.user_data_set({
        'state': state,
        'tempsensor': sensor_topic,
        'target_temp': conf.get('heating', 'target_temp_topic'),
        'thermostat_status_topic': conf.get('heating',
                                            'thermostat_status_topic'),
        'thermostat_id': thermostat_id,
        })
    mqttc.on_connect = on_connect
    mqttc.on_message = on_message
    mqttc.username_pw_set(conf.get('mqtt', 'user'),
                          conf.get('mqtt', 'password'))
    mqttc.will_set(
        conf.get('heating', 'thermostat_status_topic'),
        json.dumps({'thermostat_id': thermostat_id, 'status': 'offline'}))
    mqttc.connect(conf.get('mqtt', 'host'), 1883, 60)

    mqttc.loop_start()
    while True:
        time.sleep(1)
        state.now = datetime.datetime.now()
        period(state)

def main():
    parser = argparse.ArgumentParser(description="Maintain target temperature")
    parser.add_argument("-n", action="store_true", dest="dry_run")
    parser.add_argument("sensor_topic")
    parser.add_argument("thermostat_id")
    args = parser.parse_args()

    maintain_temp(args.sensor_topic, int(args.thermostat_id, 0), args.dry_run)

if __name__ == "__main__":
    main()



""" Interface to scheduler database. """

import psycopg2

def db_connect(host, db, user, pw):
    """ Connect to database

    Returns a postgres database connection created using the provided credentials.
    """
    connect_string = 'host={} dbname={} user={} password={}'.format(
        host, db, user, pw)
    conn = psycopg2.connect(connect_string)
    return conn

def update_last_temperature(db, when, temp):
    """ Update cached temperature value. """
    cursor = db.cursor()
    cursor.execute('delete from temperature_cache;')
    cursor.execute('insert into temperature_cache '
                   '(temperature, updated) values (%s, %s);',
                   (temp, when))

def get_last_temperature(db):
    """ Return cached temperature value. """
    cursor = db.cursor()
    cursor.execute('select temperature, updated from temperature_cache '
                   'limit 1;')
    results = cursor.fetchall()
    if len(results) == 1:
        r1 = results[0]
        return TempReading(r1[1], r1[0])
    else:
        return None

class FullSchedule(object):
    """ The heating schedule. """
    def __init__(self, entries):
        self.entries = entries

    def __iter__(self):
        return self.entries.__iter__()

    @classmethod
    def create_entry(cls, db, dow, time, temp):
        """ Create a schedule entry in the database.

        dow is the day of week, counting from 0 being Monday. """
        cursor = db.cursor()
        cursor.execute("insert into schedule (day, starttime, temp) "
                       "values (%s, %s, %s)", (dow, time, temp))

    @classmethod
    def delete_entry(cls, db, dow, time):
        """ Remove an entry form the databse.

        dow is the day of week, counting from 0 being Monday. """
        cursor = db.cursor()
        cursor.execute("delete from schedule where day=%s and starttime=%s",
                       (dow, time))

    @classmethod
    def from_db(cls, db):
        """ Create a schedule class instance from the database. """
        cursor = db.cursor()
        cursor.execute("select day, starttime, temp from schedule "
                       "order by day, starttime")
        entries = []
        for record in cursor:
            entries.append(record)
        return cls(entries)

class TempReading(object):
    """ Convenience class representing a temperature reading. """
    def __init__(self, when, temp):
        self.when = when
        self.temp = temp

class TargetOverride(object):
    """ Override the set temperature for a period of time. """
    def __init__(self, end, temp):
        self.end = end
        self.temp = temp

    @classmethod
    def from_db(cls, connection):
        cursor = connection.cursor()
        cursor.execute('select until, temp from override limit 1')
        data = cursor.fetchone()
        if not data:
            return None
        until, temp = data
        return cls(until, temp)

    @classmethod
    def clear_from_db(cls, connection):
        """ Cancel any current override in database. """
        cursor = connection.cursor()
        cursor.execute('delete from override')

    def save(self, connection):
        """ Replaces the current override in the database with this. """
        cursor = connection.cursor()
        cursor.execute('delete from override')
        cursor.execute('insert into override (until, temp) '
                       'values (%s, %s)',
                       (self.end, self.temp))




#!/usr/bin/env python

"""Operate heating schedule."""

import sys
import json
import datetime
import threading
import logging
import paho.mqtt.client as mqtt
import requests
import requests.exceptions
from requests.auth import HTTPBasicAuth

import model
import config

logging.basicConfig()
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

strptime = datetime.datetime.strptime
strftime = datetime.datetime.strftime

class SchedulerTemperaturePolicy(object):
    ENTRY_TARGET_OVERRIDE = -2

    def __init__(self, schedule, tgt_override):
        self.schedule = schedule
        self.target_override = tgt_override

    @classmethod
    def from_json(cls, j):
        data = json.loads(j)

        entries = []
        for day in sorted(data['schedule']):
            for item in data['schedule'][day]:
                entries.append((
                    int(day),
                    strptime(item['time'], "%H:%M").time(),
                    item['temp']
                    ))
        schedule = model.FullSchedule(entries)
        tgt_override = data['target_override']
        if all(x in tgt_override for x in ['temp', 'until']):
            tgt_override_obj = model.TargetOverride(
                strptime(tgt_override['until'], '%Y-%m-%dT%H:%M'),
                tgt_override['temp'])
        else:
            tgt_override_obj = None
        return cls(schedule, tgt_override_obj)

    def get_day(self, day):
        """Determine the schedule for today covering the full 24h.

        Uses the overall schedule and any schedule overrides (but not
        target tmperature overrides) in place.
        """
        if len(self.schedule.entries) == 0:
            return []
        entries = []

        # Populate with the day's entries.  Add an entry from the start
        # of the day based on entry prior to that day:
        candidate_beginning = None
        for sday, starttime, temp in self.schedule.entries:
            if sday == day:
                entries.append((starttime, temp))
            elif sday < day:
                candidate_beginning = temp
        if candidate_beginning is None:
            # Wasn't an earlier entry, use the last one in the schedule:
            _, _, temp = self.schedule.entries[-1]
            candidate_beginning = temp
        # Check whether we need to fill in the start:
        if len(entries) == 0 or entries[0][0] != datetime.time(0, 0):
            entries.insert(0, (datetime.time(0, 0), candidate_beginning))

        return entries

    def target(self, now):
        """Determine temperature target at datetime now.

        Returns a pair of (target, index) where index is the number of the
        entry in the day's schedule returned by get_day. Index is -2 if from a
        target override.
        """
        # First check if we are still within an override:
        if self.target_override is not None:
            if self.target_override.end > now:
                return (self.target_override.temp,
                        self.ENTRY_TARGET_OVERRIDE)

        day_schedule = self.get_day(now.weekday())
        now_time = now.time()

        # Iterate over the schedule finding the last entry with a lower time
        # than the requested time.  If we don't find one, we need the last
        # entry from the day before.
        entry = -1
        target = None
        for sched_time, sched_target in day_schedule:
            if sched_time <= now_time:
                target = sched_target
                entry += 1
            else:
                break
        if target is None:
            entry = None
        return target, entry

def mqtt_on_connect(client, userdata, flags, rc):
    if rc:
        logger.error("Error connecting, rc %d", rc)
        return
    client.subscribe(userdata['thermostat_schedule_change_topic'])
    client.subscribe(userdata['thermostat_status_topic'])
    client.subscribe(userdata['temperature_sensor_topic'])

def mqtt_on_message(client, userdata, msg):
    if msg.topic == userdata['thermostat_schedule_change_topic']:
        userdata['timer_event'].set()
    elif msg.topic == userdata['thermostat_status_topic']:
        if json.loads(msg.payload)['status'] == 'online':
            userdata['timer_event'].set()
    elif msg.topic == userdata['temperature_sensor_topic']:
        data = json.loads(msg.payload)
        if 'temperature' in data:
            try:
                temp = float(data['temperature'])
                now = strftime(datetime.datetime.now(), "%Y-%m-%dT%H:%M:%S")
                r = requests.post(
                    userdata['scheduler_url'] + '/temperature',
                    auth=userdata['auth'],
                    timeout=10, data={
                        'when': now,
                        'temp': temp,
                    })
                logger.info("Cached temperature update %.2f, result %d",
                            temp, r.status_code)
            except (requests.exceptions.RequestException, ValueError) as e:
                logger.info("Error updating cached temperature (%s)",
                            str(e))

def scheduler_iteration(mqttc, target_temp_topic, scheduler_url, auth, now):
    """Fetch schedule, determine target and set it.

    Has no return value and swallows any exceptions fetching the schedule,
    etc., with the intention that a daemon can use this function in a simple
    loop and the correct behaviour will result."""
    try:
        r = requests.get(scheduler_url + "/schedule", auth=auth, timeout=10)
    except requests.exceptions.RequestException as e:
        logger.error("Failed interval (%s)", str(e))
        return

    if r.status_code != 200:
        logger.error("Couldn't get schedule (%d)", r.status_code)
        return

    scheduler = SchedulerTemperaturePolicy.from_json(r.text)
    target = scheduler.target(now)
    if target[0] is None:
        logger.info("No target temperature available.")
    else:
        logger.info("Publishing target temperature update: %d", target[0])
        mqttc.publish(target_temp_topic, json.dumps({'target': target[0]}))

def main():
    conf = config.load_config()

    # If the 'heating' section of the config has a 'scheduler_username' and
    # 'scheduler_password' entry, we use these with HTTP basic auth.
    if conf.has_option('heating', 'scheduler_username') and \
       conf.has_option('heating', 'scheduler_password'):
        logger.info("Using HTTP basic authentication")
        auth = HTTPBasicAuth(conf.get('heating', 'scheduler_username'),
                             conf.get('heating', 'scheduler_password'))
    else:
        logger.info("Not using HTTP authentication")
        auth = None

    if len(sys.argv) == 2:
        scheduler_url = sys.argv[1]
    else:
        scheduler_url = conf.get('heating', 'scheduler_url')

    period_event = threading.Event()
    mqttc = mqtt.Client(userdata={
        'conf': conf,
        'thermostat_status_topic':
            conf.get('heating', 'thermostat_status_topic'),
        'thermostat_schedule_change_topic':
            conf.get('heating', 'thermostat_schedule_change_topic'),
        'temperature_sensor_topic':
            conf.get('heating', 'temperature_sensor_topic'),
        'timer_event': period_event,
        'scheduler_url': scheduler_url,
        'auth': auth,
        })
    mqttc.username_pw_set(conf.get('mqtt', 'user'),
                          conf.get('mqtt', 'password'))
    mqttc.on_connect = mqtt_on_connect
    mqttc.on_message = mqtt_on_message
    mqttc.connect(conf.get('mqtt', 'host'), 1883, 60)

    mqttc.loop_start()
    while True:
        scheduler_iteration(mqttc, conf.get('heating', 'target_temp_topic'),
                            scheduler_url, auth, datetime.datetime.now())
        period_event.wait(timeout=60)
        period_event.clear()

    mqttc.loop_stop()

if __name__ == "__main__":
    main()



import subprocess
from setuptools import setup

# Try to create an rst long_description from README.md:
try:
    args = 'pandoc', '--to', 'rst', 'README.md'
    long_description = subprocess.check_output(args)
    long_description = long_description.decode()
except Exception as error:
    print("WARNING: Couldn't generate long_description - is pandoc installed?")
    long_description = None

setup(name='boilerio',
      version='0.0.1',
      description='A software thermostat and heating control system',
      url='https://github.com/adpeace/boilerio.git',
      author='Andy Peace',
      author_email='andrew.peace@gmail.com',
      license='MIT',
      long_description=long_description,
      classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'License :: OSI Approved :: MIT License',
        ],
      packages=['boilerio'],
      scripts=['bin/boiler_to_mqtt', 'bin/boilersim'],
      entry_points={'console_scripts': [
        'scheduler=boilerio.scheduler:main',
        'maintaintemp=boilerio.maintaintemp:main',
        ]},
      setup_requires=['pytest-runner'],
      install_requires=['Flask', 'requests', 'psycopg2', 'paho-mqtt'],
      tests_require=['pytest', 'requests-mock', 'mock'],
      zip_safe=False)
